"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var immer_1 = require("immer");
var StateReturnRequiredError_1 = require("../exceptions/StateReturnRequiredError");
var store_1 = require("../utils/store");
var persist_1 = require("../utils/persist");
var BaseReducer = /** @class */ (function () {
    function BaseReducer(init, instanceName, filterPersistData) {
        this.cases = [];
        this.initData = init;
        this.reducerName = instanceName;
        this.filterPersistData = filterPersistData;
    }
    BaseReducer.prototype.clear = function () {
        this.cases = [];
    };
    BaseReducer.prototype.addCase = function () {
        var _a;
        var config = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            config[_i] = arguments[_i];
        }
        (_a = this.cases).push.apply(_a, tslib_1.__spread(config));
    };
    BaseReducer.prototype.getData = function () {
        return store_1.getState()[this.reducerName];
    };
    BaseReducer.prototype.createData = function () {
        var _a;
        var _this = this;
        return _a = {},
            _a[this.reducerName] = function (state, action) {
                var e_1, _a;
                if (state === undefined) {
                    var newState = persist_1.switchInitData(_this.reducerName, _this.initData);
                    return newState === _this.initData ? newState : _this.persistState(newState);
                }
                // For async storage, we should dispatch action to inject persist data into reducer
                if (action.type === persist_1.TYPE_REHYDRATE && action.payload[_this.reducerName] !== undefined) {
                    return _this.persistState(action.payload[_this.reducerName]);
                }
                // Actions
                if (action.reducerName === _this.reducerName) {
                    if (action.effect) {
                        return _this.changeState(action.effect, state, action);
                    }
                }
                else if (_this.cases.length) {
                    try {
                        // Effects
                        for (var _b = tslib_1.__values(_this.cases), _c = _b.next(); !_c.done; _c = _b.next()) {
                            var _d = _c.value, when = _d.when, effect = _d.effect;
                            if (when === action.type) {
                                return _this.changeState(effect, state, action);
                            }
                        }
                    }
                    catch (e_1_1) { e_1 = { error: e_1_1 }; }
                    finally {
                        try {
                            if (_c && !_c.done && (_a = _b.return)) _a.call(_b);
                        }
                        finally { if (e_1) throw e_1.error; }
                    }
                }
                return state;
            },
            _a;
    };
    BaseReducer.prototype.persistState = function (state) {
        if (!this.filterPersistData) {
            return state;
        }
        if (immer_1.isDraftable(state)) {
            var draft = immer_1.createDraft(state);
            var responseDraft = this.filterPersistData(draft);
            if (responseDraft === undefined) {
                state = immer_1.finishDraft(draft);
            }
            else if (immer_1.isDraft(responseDraft)) {
                state = immer_1.finishDraft(responseDraft);
            }
            else {
                state = responseDraft;
            }
        }
        else {
            state = this.filterPersistData(state);
            if (state === undefined) {
                throw new StateReturnRequiredError_1.StateReturnRequiredError('filterPersistData');
            }
        }
        return state;
    };
    ;
    BaseReducer.prototype.changeState = function (effect, state, action) {
        if (immer_1.isDraftable(state)) {
            var draft = immer_1.createDraft(state);
            var responseDraft = effect(draft, action);
            if (responseDraft === undefined) {
                state = immer_1.finishDraft(draft);
            }
            else if (immer_1.isDraft(responseDraft)) {
                state = immer_1.finishDraft(responseDraft);
            }
            else {
                state = responseDraft;
            }
        }
        else {
            state = effect(state, action);
            if (state === undefined) {
                throw new StateReturnRequiredError_1.StateReturnRequiredError(action.type);
            }
        }
        return state;
    };
    return BaseReducer;
}());
exports.BaseReducer = BaseReducer;
