"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var diff = tslib_1.__importStar(require("deep-diff"));
var lodash_clonedeep_1 = tslib_1.__importDefault(require("lodash.clonedeep"));
var PATCH_KIND;
(function (PATCH_KIND) {
    PATCH_KIND["array"] = "A";
    PATCH_KIND["new"] = "N";
    PATCH_KIND["trash"] = "D";
    PATCH_KIND["edit"] = "E";
})(PATCH_KIND || (PATCH_KIND = {}));
var getDeepState = function (vmState, paths, popLastPath) {
    if (popLastPath) {
        paths = paths.slice(0, paths.length - 1);
    }
    return paths.reduce(function (carry, value) {
        return carry[value];
    }, vmState);
};
var getLastPath = function (paths) {
    return paths.slice(paths.length - 1)[0];
};
exports.applyPatch = function (original, current, vmState) {
    var patches = diff.diff(original, current);
    if (patches === undefined) {
        return;
    }
    patches.forEach(function (patch) {
        var patchPath = patch.path;
        switch (patch.kind) {
            case PATCH_KIND.array:
                if (patch.item.kind !== PATCH_KIND.new) {
                    throw new TypeError("Unknown diff kind '" + patch.item.kind + "' in array");
                }
                getDeepState(vmState, patchPath, false)[patch.index] = lodash_clonedeep_1.default(patch.item.rhs);
                break;
            case PATCH_KIND.trash:
                Reflect.deleteProperty(getDeepState(vmState, patchPath, true), getLastPath(patchPath));
                break;
            case PATCH_KIND.edit:
            case PATCH_KIND.new:
                getDeepState(vmState, patchPath, true)[getLastPath(patchPath)] = lodash_clonedeep_1.default(patch.rhs);
                break;
            default:
                throw new TypeError('Unknown diff kind: ' + patch.kind);
        }
    });
};
